/*
** Copyright (c) 2003-2005, Oracle Corporation. All Rights Reserved.
** Diagrammer related functions
*/

function GraphView(source, mainImageID, form,
                   graphBounds,
                   thumbWidth, thumbHeight,
                   viewClip)
{
  this.source = source;
  this.imageID = mainImageID;
  this.graphBounds = graphBounds;
  this.viewClip = viewClip;

  this.formName = form;
  this.thumbWidth = thumbWidth;
  this.thumbHeight = thumbHeight;
}

GraphView.prototype.submit = _submit;
GraphView.prototype.move = _move;
GraphView.prototype.moveRel = _moveRel;
GraphView.prototype.thumbClick = _thumbClick;
GraphView.prototype.nodeClick = _nodeClick;
GraphView.prototype.canvasClick = _canvasClick;

function _canvasClick(eventObj)
{
  var imgTag = document.getElementById(this.imageID);
  var pt = _getClickXY(imgTag, eventObj);
  var vc = this.viewClip;

  var p = new Object();
  p.event = "canvas";
  p.x = pt.x + vc.x;
  p.y = pt.y + vc.y;
  this.submit(p);
}

function _nodeClick(id,part)
{
  var p = new Object();
  p.event = "click";
  p.part = part;
  p.partID = id;
  this.submit(p);
}

function _moveRel(x,y)
{
  var vc = this.viewClip;
  var xx = (vc.x + vc.width / 2) + (vc.width * x / 2);
  var yy = (vc.y + vc.height / 2) + (vc.height * y / 2);
  this.move(xx, yy);
}

/**
 * @param x, y the center of the new view port
 */
function _move(x,y)
{
  var p = new Object();
  p.event = "pan";
  p.x = x;
  p.y = y;
  this.submit(p);
}

function _submit(p)
{
  p.source = this.source;
  submitForm(this.formName,0,p);
}

function _thumbClick(tag, eventObj)
{
  var pt = _getClickXY(tag, eventObj);
  var xx = pt.x;
  var yy = pt.y;

  //alert(eventObj.clientX);

  var gb = this.graphBounds;

  var x = (xx * gb.width / this.thumbWidth) + gb.x;
  var y = (yy * gb.height / this.thumbHeight) + gb.y;
  this.move(Math.round(x), Math.round(y));
}

function _getClickXY(tag, eventObj)
{
  var pt = _getLocation(tag);
  pt.x = eventObj.clientX - pt.x;
  pt.y = eventObj.clientY - pt.y;
  return pt;
}

function _getLocation(pos)
{
 var oCurrentNode=pos;
 var rootx=0;
 var rooty=0;
 while(oCurrentNode.tagName!="BODY") {
   // alert(oCurrentNode.tagName+" y:"+rooty);
      rootx+=oCurrentNode.offsetLeft;
      rooty+=oCurrentNode.offsetTop;
      oCurrentNode=oCurrentNode.offsetParent;
 }

 return {x:rootx,y:rooty};
}
